<?php

namespace Ulmus;

use Psr\SimpleCache\CacheInterface;
use Ulmus\Adapter\AdapterInterface;

use Ulmus\Common\PdoObject;

class ConnectionAdapter
{
    protected AdapterInterface $adapter;

    protected PdoObject $pdo;

    public function __construct(
        public string $name = "default",
        protected array $configuration = [],
        public bool $default = false,
        public ? CacheInterface $cacheObject = null
    ) {
        Ulmus::registerAdapter($this, $default);
    }

    public function resolveConfiguration() : void
    {
        $connection = $this->configuration['connections'][$this->name] ?? [];

        if ( false !== ( $adapterName = $connection['adapter'] ?? false ) ) {
            $this->adapter = $this->instanciateAdapter($adapterName);
        }
        else {
            throw new \InvalidArgumentException(sprintf("Adapter not found within your configuration array. (%s)", json_encode($connection)));
        }
        
        $this->adapter->setup($connection);

        unset($this->configuration['connections']);
    }

    public function getConfiguration() : array
    {
        return $this->configuration['connections'][$this->name] ?? [];
    }

    /**
     * Connect the adapter
     * @return self
     */
    public function connect() : self
    {
        $this->pdo = $this->adapter->connect();

        return $this;
    }
    
    public function adapter() : AdapterInterface
    {
        return $this->adapter;
    }
    
    public function pdo() : PdoObject
    {
        return $this->pdo ?? $this->connect()->pdo;
    }

    public function connector() : object
    {
        return $this->pdo();
    }
    
    /**
     * Instanciate an adapter which interact with the data source
     * @param  string $name An Ulmus adapter or full class name implementing AdapterInterface
     * @return AdapterInterface
     */
    protected function instanciateAdapter(string $name) : AdapterInterface
    {
        $class = substr($name, 0, 2) === "\\" ? $name : sprintf("\\%s\\Adapter\\%s", __NAMESPACE__, $name);
        
        return new $class();
    }
}
